@@ -5,11 +5,11 @@ See the Manifest class.
 """
 
 from collections.abc import Sequence
+import multiprocessing
 import os
 from typing import Dict
 from typing import List
 from typing import Optional
-from typing import Tuple
 
 import hyperthought as ht
 from hyperthought.metadata import MetadataItem
@@ -26,22 +26,29 @@ class JobDataNotFoundException(Exception):
     pass
 
 
-def _compute_hash_and_end_bytes(path: str) -> Tuple[str, List[int]]:
+def _compute_size_hash_and_end_bytes(file_: Dict) -> Dict:
     """
-    Compute file hash and end bytes for a file.
+    Compute size, hash, and end bytes for a file.
 
     Parameters
     ----------
-    path : str
-        Path to the file of interest.
+    file_ : dict
+        Database record for the file of interest.
 
     Returns
     -------
-    A tuple containing hash and end_bytes for the file.
+    A dict with keys "id", "size", "hash", and "end_bytes".
     """
-    hash = utils.get_hash(file_name=path)
-    end_bytes = utils.get_end_bytes(path=path)
-    return hash, end_bytes
+    path = file_["path"]
+    size = os.path.getsize(path)
+    file_hash = utils.get_hash(file_name=path)
+    end_bytes = ",".join(str(i) for i in utils.get_end_bytes(path=path))
+    return {
+        "id": file_["id"],
+        "size": size,
+        "file_hash": file_hash,
+        "end_bytes": end_bytes,
+    }
 
 
 class Manifest:
@@ -109,6 +116,7 @@ class Manifest:
         self._hyperthought_root_id_path = None
         self._database_connection = None
         self._database = None
+        self._file_info_computed = False
 
         if os.path.exists(self._manifest_file):
             if not os.path.isfile(manifest_file):
@@ -183,23 +191,65 @@ class Manifest:
         job_data = self._database.get_job_data()
 
         if job_data is None:
-            raise JobDataNotFoundException(f"No job data in {self._manifest_file}")
+            raise JobDataNotFoundException(
+                f"No job data in {self._manifest_file}")
 
         self._job_name = job_data["job_name"]
         self._username = job_data["username"]
         self._workspace_alias = job_data["workspace_alias"]
         self._ignore_path = job_data["ignore_path"]
         self._hyperthought_root = job_data["hyperthought_root"]
-        root_id = self._database.get_file_id(path=self._hyperthought_root)
 
-        if root_id is None:
-            self._create_hyperthought_root()
-        else:
-            root = self._database.get_file(file_id=root_id)
-            self._hyperthought_root_id_path = (
-                f"{root['hyperthought_id_path']}" f"{root['hyperthought_id']},"
+        stripped_root = self._hyperthought_root.strip("/")
+
+        if not stripped_root:
+            return
+
+        cursor = self._database_connection.cursor()
+
+        def get_hyperthought_id(name, hyperthought_id_path):
+            # TODO:  Remove SQL from this module.  See IC-????.
+            sql = """
+            SELECT hyperthought_id
+            FROM File
+            WHERE
+                name = :name
+                AND
+                hyperthought_id_path = :hyperthought_id_path
+            """
+            cursor.execute(sql, {
+                "name": name,
+                "hyperthought_id_path": hyperthought_id_path,
+            })
+            row = cursor.fetchone()
+
+            if not row:
+                return None
+
+            return row["hyperthought_id"]
+
+        root_folders = stripped_root.split("/")
+        hyperthought_id_path = ","
+
+        for folder_name in root_folders:
+            hyperthought_id = get_hyperthought_id(
+                name=folder_name,
+                hyperthought_id_path=hyperthought_id_path,
             )
 
+            if not hyperthought_id:
+                hyperthought_id = utils.generate_id()
+                self._database.create_file(
+                    name=folder_name,
+                    is_folder=True,
+                    hyperthought_id=hyperthought_id,
+                    hyperthought_id_path=hyperthought_id_path,
+                )
+
+            hyperthought_id_path += hyperthought_id + ","
+
+        self._hyperthought_root_id_path = hyperthought_id_path
+
     def _validate_job_data(self) -> None:
         """
         Validate job data parameters (to constructor).
@@ -212,13 +262,26 @@ class Manifest:
         if not self._username or not isinstance(self._username, str):
             raise ValueError("username must be a non-empty string")
 
-        if not self._workspace_alias or not isinstance(self._workspace_alias, str):
+        if (
+            not self._workspace_alias
+            or
+            not isinstance(self._workspace_alias, str)
+        ):
             raise ValueError("workspace_id must be a non-empty string")
 
-        if self._ignore_path is not None and not os.path.isdir(self._ignore_path):
-            raise ValueError("ignore_path_prefix must be a directory if provided")
+        if (
+            self._ignore_path is not None
+            and
+            not os.path.isdir(self._ignore_path)
+        ):
+            raise ValueError(
+                "ignore_path_prefix must be a directory if provided")
 
-        if not self._hyperthought_root or not isinstance(self._hyperthought_root, str):
+        if (
+            not self._hyperthought_root
+            or
+            not isinstance(self._hyperthought_root, str)
+        ):
             raise ValueError("hyperthought_root must be a string")
 
         if not self._hyperthought_root.startswith("/"):
@@ -226,7 +289,8 @@ class Manifest:
 
     @property
     def common_metadata(self) -> List[MetadataItem]:
-        return ht.metadata.from_api_format(self._database.get_common_metadata())
+        return ht.metadata.from_api_format(
+            self._database.get_common_metadata())
 
     @common_metadata.setter
     def common_metadata(self, metadata: List[MetadataItem]) -> None:
@@ -245,6 +309,10 @@ class Manifest:
     def hyperthought_root_id_path(self) -> str:
         return self._hyperthought_root_id_path
 
+    @property
+    def file_info_computed(self) -> bool:
+        return self._file_info_computed
+
     def _validate_metadata(self, metadata: List[MetadataItem]) -> None:
         """
         Make sure metadata is a list of MetadataItem objects.
@@ -274,17 +342,10 @@ class Manifest:
         if self.has_path(path):
             return
 
-        # Compute file info.
-        # Do this before creating any records in case there's an error.
         is_folder = os.path.isdir(path)
 
-        if is_folder:
-            file_hash = None
-            end_bytes = None
-            size = None
-        else:
-            file_hash, end_bytes = _compute_hash_and_end_bytes(path=path)
-            size = os.path.getsize(path)
+        if not is_folder:
+            self._file_info_computed = False
 
         # Get a list of ancestor folders.
         len_ignorepath = (
@@ -326,7 +387,8 @@ class Manifest:
             ancestor_id = self._database.get_file_id(path=ancestor_path)
             ancestor = self._database.get_file(file_id=ancestor_id)
             ht_id_path = (
-                f"{ancestor['hyperthought_id_path']}" f"{ancestor['hyperthought_id']},"
+                f"{ancestor['hyperthought_id_path']}"
+                f"{ancestor['hyperthought_id']},"
             )
         else:
             ht_id_path = self.hyperthought_root_id_path
@@ -366,15 +428,35 @@ class Manifest:
                 "hyperthought_id_path": ht_id_path,
                 "is_folder": is_folder,
                 "path": path,
-                "end_bytes": end_bytes,
-                "size": size,
-                "file_hash": file_hash,
+                "end_bytes": None,
+                "size": None,
+                "file_hash": None,
             }
         )
 
         # Create all records.
         self._database.bulk_load_files(file_data=file_data)
 
+    def remove_path(self, path: str) -> None:
+        """
+        Remove a path from the manifest.
+
+        Parameters
+        ----------
+        path : str
+            The path to be removed.
+        """
+        ids_to_remove = [self.get_path_id(path)]
+
+        if os.path.isdir(path):
+            ids_to_remove.extend(
+                file_["id"]
+                for file_ in self.get_progeny(folder_path=path)
+            )
+
+        for id_ in ids_to_remove:
+            self._database.delete_file(file_id=id_)
+
     def _validate_path(self, path: str) -> None:
         """
         Determine whether a path is valid.
@@ -388,8 +470,13 @@ class Manifest:
                 f"path '{path}' does not begin with " f"'{self.ignore_path}'"
             )
 
-        if not os.path.isfile(path) and not os.path.isdir(path):
-            raise ValueError("path must be a file or directory")
+        if (
+            os.path.islink(path)
+            or
+            not (os.path.isfile(path) or os.path.isdir(path))
+        ):
+            raise ValueError(
+                "path must be a file or directory (links not allowed)")
 
     def has_path(self, path: str) -> bool:
         """Determine whether a path has already been added."""
@@ -500,8 +587,67 @@ class Manifest:
         # TODO:  Implement.  See IC-7229.
         pass
 
-    def close(self):
-        """Close the manifest (database)."""
+    def _compute_file_info_with_pool(
+        self,
+        n_processes: Optional[int] = None,
+    ) -> None:
+        """Compute file size, hash, and end bytes using a process pool."""
+        if (
+            n_processes is not None
+            and
+            not (isinstance(n_processes, int) and n_processes > 0)
+        ):
+            raise ValueError(
+                "n_processes must be a positive integer if provided")
+
+        if n_processes is None:
+            n_processes = multiprocessing.cpu_count() * 2
+
+        files = self._database.get_all_files()
+        pool = multiprocessing.Pool(processes=n_processes)
+        all_updates = pool.map(_compute_size_hash_and_end_bytes, files)
+        pool.close()
+        pool.join()
+
+        for updates in all_updates:
+            id_ = updates.pop("id")
+            self._database.update_file(id_=id_, updates=updates)
+
+    def _compute_file_info_without_pool(self) -> None:
+        """Compute file hashes and end bytes in the current process."""
+        files = self._database.get_all_files()
+
+        for file_ in files:
+            updates = _compute_size_hash_and_end_bytes(file_=file_)
+            id_ = updates.pop("id")
+            self._database.update_file(id_=id_, updates=updates)
+
+    def compute_file_info(
+        self,
+        use_pool: bool = True,
+        n_processes: Optional[int] = None,
+    ) -> None:
+        """Compute hash and end_bytes for each file in the manifest."""
+        if use_pool:
+            self._compute_file_info_with_pool(n_processes=n_processes)
+        else:
+            self._compute_file_info_without_pool()
+
+        self._file_info_computed = True
+
+    def close(self, compute_file_info: bool = True) -> None:
+        """
+        Close the manifest (database).
+
+        Parameters
+        ----------
+        compute_file_info : bool
+            If True, update file records to include sizes, hashes,
+            and end bytes.
+        """
+        if compute_file_info and not self.file_info_computed:
+            self.compute_file_info()
+
         self._database_connection.close()
 
 
@@ -561,9 +707,11 @@ def from_json(manifest_data: Dict, manifest_database_file: str) -> Manifest:
 
     # Add file-specific metadata.
     for metadata_item in manifest_data["metadata"]["fileSpecific"]:
-        metadata = ht.metadata.from_api_format(metadata=metadata_item["metadata"])
+        metadata = ht.metadata.from_api_format(
+            metadata=metadata_item["metadata"])
         apply_to_file_ids = [
-            json_id_to_database_id[id_] for id_ in metadata_item["applyToFileIds"]
+            json_id_to_database_id[id_]
+            for id_ in metadata_item["applyToFileIds"]
         ]
         manifest.add_metadata(
             metadata=metadata,
