@@ -166,7 +166,8 @@ class DatabaseManager:
             invalid_chars = set(end_bytes) - valid_chars
 
             if invalid_chars:
-                raise ValueError(f"Invalid chars in end_bytes: {invalid_chars}")
+                raise ValueError(
+                    f"Invalid chars in end_bytes: {invalid_chars}")
 
         # Define and execute the relevant SQL statement.
         sql = """
@@ -213,6 +214,25 @@ class DatabaseManager:
 
         return file_id
 
+    def delete_file(self, file_id: int, commit: bool = True) -> None:
+        """
+        Remove a file record from the database.
+
+        Parameters
+        ----------
+        file_id : int
+            The id of the file to be deleted.
+        commit : bool
+            Determines whether changes will be committed to the database.
+            This may be false if the changes are part of a transaction.
+        """
+        cursor = self._conn.cursor()
+        sql = "DELETE FROM File WHERE id = :id"
+        cursor.execute(sql, {"id": file_id})
+
+        if commit:
+            self._conn.commit()
+
     def create_parser(
         self,
         file_id: int,
@@ -348,7 +368,8 @@ class DatabaseManager:
 
         for item in metadata:
             if not isinstance(item, ht.metadata.MetadataItem):
-                raise ValueError("all metadata items must be instances of MetadataItem")
+                raise ValueError(
+                    "all metadata items must be instances of MetadataItem")
 
         api_metadata = ht.metadata.to_api_format(metadata=metadata)
         cursor = self._conn.cursor()
@@ -673,7 +694,13 @@ class DatabaseManager:
         sql = "SELECT * FROM File WHERE path LIKE :path_pattern"
         path_pattern = f"{folder_path.rstrip(os.path.sep)}{os.path.sep}%"
         cursor.execute(sql, {"path_pattern": path_pattern})
-        return [{key: row[key] for key in row.keys()} for row in cursor.fetchall()]
+        return [
+            {
+                key: row[key]
+                for key in row.keys()
+            }
+            for row in cursor.fetchall()
+        ]
 
     def update_parser(
         self,
@@ -758,7 +785,7 @@ class DatabaseManager:
                 path : str
                     Local path to the file or folder.
                     Can be None (for folders that don't exist locally).
-                end_bytes : str
+                end_bytes : str or None
                     Comma-separated ints representing bytes found at the end
                     of a file.
                 size : int or None
@@ -791,7 +818,8 @@ class DatabaseManager:
             missing_keys = required_keys - keys
 
             if missing_keys:
-                raise ValueError(f"Element in file_data missing keys: {missing_keys}")
+                raise ValueError(
+                    f"Element in file_data missing keys: {missing_keys}")
 
             item["id"] = next_id
             next_id += 1
@@ -864,7 +892,8 @@ class DatabaseManager:
             missing_keys = required_keys - keys
 
             if missing_keys:
-                raise ValueError(f"Element in parser_data missing keys: {missing_keys}")
+                raise ValueError(
+                    f"Element in parser_data missing keys: {missing_keys}")
 
             item["id"] = next_id
             next_id += 1
@@ -917,7 +946,8 @@ class DatabaseManager:
 
         for item in metadata_data:
             if not isinstance(item, dict):
-                raise ValueError("All elements of metadata_data must be dicts.")
+                raise ValueError(
+                    "All elements of metadata_data must be dicts.")
 
             keys = set(item.keys())
             missing_keys = required_keys - keys
@@ -1074,7 +1104,8 @@ class DatabaseManager:
             sql,
             {
                 "hash_mismatch_status_id": FileStatus.HASH_MISMATCH.value,
-                "malware_detected_status_id": (FileStatus.MALWARE_DETECTED.value),
+                "malware_detected_status_id": (
+                    FileStatus.MALWARE_DETECTED.value),
                 "other_error_status_id": FileStatus.UNKNOWN_ERROR.value,
             },
         )
